﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class PaddleLogic : MonoBehaviour {

	public static PaddleLogic instance;
	public GameObject surfer;
	public Animator waterAnimator;
	public float waterspeed = 0.1f;
	public float speed_increase = 0.1f;
	public static float max_swipe_length = 100f;
	public float speed_required = 1f;
	public Text timer_text;
	public Text message_text;

	Stroke currentstroke;
	Stroke laststroke;
	Vector2 speedboost = Vector2.zero;

	public class Stroke{
		public Stroke(Vector2 startpoint, bool isrightside){
			this.start = startpoint;
			this.side = isrightside;
		}
		/** isrightside*/
		public bool side;
		public Vector2 start;
		public Vector2 stop;

        internal Vector2 CalcDirection(){
            return (start-stop).normalized; // direction of force is opposite of swipe direction
        }
		internal float CalcStrength(){
            float magnitude = (start-stop).magnitude;
			// cap impulse
			if (magnitude > max_swipe_length){
				return 1f;
			}
			return magnitude / max_swipe_length;
        }

        internal void Finish(Vector2 position){
            stop = position;
			Debug.Log("Stroke finished: "+start+"-"+stop+"="+CalcDirection()+" * "+CalcStrength());
        }
    }

	public float initialDistance;
	GameObject s1;
	GameObject s2;
	void Awake(){
		instance = this;
	}

	void Start(){
		if (GameData.instance == null){
			// for debugging
			this.gameObject.AddComponent(Type.GetType( "GameData" ));
		}

		GameData.instance.timePassed = 0;
		GameData.instance.distanceRemaining = initialDistance;
		s1 = surfer.transform.Find("sprite1").gameObject;
		s2 = surfer.transform.Find("sprite2").gameObject;
	}
	void Update () {
		if (!GameLogic.instance.tutorialIsRunning){
			UpdateTimer();
		}
		
		Vector3 surferScreenPos = Camera.main.WorldToScreenPoint(surfer.transform.position);

// non-touchdevice version
        if(Input.GetMouseButtonDown(0) && currentstroke == null) {
			//Debug.Log("mouse down");
			currentstroke = new Stroke(Input.mousePosition, Input.mousePosition.x > surferScreenPos.x);
        }
		if(Input.GetMouseButtonUp(0) && currentstroke != null) {
			//Debug.Log("mouse up");
			currentstroke.Finish(Input.mousePosition);
			// compare with last stroke and calc boost
			Vector2 speedboost = CalculateBoost(currentstroke, laststroke);
			// apply boost
			laststroke = currentstroke;
			currentstroke = null;
			bool applyboost = true;
			if (GameLogic.instance.tutorialIsRunning){
				if (GameLogic.instance.ContinueTutorial(laststroke) == false){
					applyboost = false;
				}
			}
			if (applyboost){
				ApplyBoost(speedboost);
			}
		}

#if UNITY_ANDROID && !UNITY_EDITOR
		if (Input.touchCount == 0 && currentstroke != null){
			//Debug.Log("no touch");
			// compare with last stroke and calc boost
			Vector2 speedboost = CalculateBoost(currentstroke, laststroke);
			// apply boost
			laststroke = currentstroke;
			currentstroke = null;
			bool applyboost = true;
			if (GameLogic.instance.tutorialIsRunning){
				if (GameLogic.instance.ContinueTutorial(laststroke) == false){
					applyboost = false;
				}
			}
			if (applyboost){
				ApplyBoost(speedboost);
			}
			ApplyBoost(speedboost);
		}
		if (Input.touchCount == 1) {
			if (currentstroke == null){
				currentstroke = new Stroke(Input.touches[0].position, Input.touches[0].position.x > surferScreenPos.x);
			}else{
				if (Input.GetTouch(0).phase == TouchPhase.Ended){
					currentstroke.Finish(Input.touches[0].position);
				}
			}
		}
#endif
	}

	private void UpdateTimer(){
        GameData.instance.timePassed += Time.deltaTime;
		timer_text.text = GameData.GetTimeAsText(GameData.instance.timePassed) + " - " + (int)(GameData.instance.distanceRemaining) + "m";

    }
	bool gave100mWarning = false;
    void FixedUpdate(){
		if (speedboost.magnitude > 0f){
			Debug.Log("slow="+speed_required+" boost="+speedboost);
		}
		surfer.GetComponent<Rigidbody2D>().AddForce(new Vector2(0, -speed_required), ForceMode2D.Force);
		surfer.GetComponent<Rigidbody2D>().AddForce(speedboost, ForceMode2D.Impulse);
		speed_required += speed_increase;
		waterAnimator.speed = speed_required*waterspeed;
		speedboost = Vector2.zero;
		if (!GameLogic.instance.tutorialIsRunning){
			GameData.instance.distanceRemaining -= speed_required*speed_required/100;
			if (GameData.instance.distanceRemaining < 100 && !gave100mWarning){
				message_text.text = "Almost there, brah! Paddle hard!!";
				gave100mWarning = true;
				StartCoroutine(CleanMessage(2f));
			}
			if (GameData.instance.distanceRemaining < 0){
				GameData.instance.distanceRemaining = 0;
				message_text.text = "You reached the swell, brah!";
				GameLogic.instance.EndScene(GameData.Outcome.Success,2f);
			}
		}
	}

	IEnumerator CleanMessage(float delay){
		yield return new WaitForSeconds(delay);
		message_text.text = "";
	} 

    private void ApplyBoost(Vector2 speedboost)
    {
		this.speedboost += speedboost;
		s1.SetActive(laststroke.side);
		s2.SetActive(!laststroke.side);
    }

    private Vector2 CalculateBoost(Stroke currentstroke, Stroke laststroke)
    {
		if (laststroke == null){
			// first stroke only half powerful
			return currentstroke.CalcDirection() * currentstroke.CalcStrength()/2;
		}
		if (currentstroke.side == laststroke.side){
			// paddling only on one side reduces impact significantly
			return (currentstroke.side ? -1 : 1) * Vector2.right * currentstroke.CalcStrength();
		}else{
			// TODO calc how nice/rythmic the stroke was
			return currentstroke.CalcDirection() * currentstroke.CalcStrength();
		}
    }
}

